# -*- coding: utf-8 -*-
"""削除されたファイルと削除された時間を記憶しておいて、
PSCがupdateした時など、ファイルを削除すべきことを
教えられるようにする。"""

__author__  = 'Kenji Saito <ksaito@do-johodai.ac.jp>'

from AccessControl import ClassSecurityInfo

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.SyncFolder.interfaces import ISyncFolder

from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import finalizeATCTSchema

from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder
import string
from DateTime import DateTime

# Copy the ATFolder schema (to avoid modifying the original) and append our
# own fields for the images and attachments manager widgets.
SyncFolderSchema = ATFolder.schema.copy()

# Finalise the schema according to ATContentTypes standards. This basically
# moves the Related items and Allow discussion fields to the bottom of the
# form. See ATContentTypes.content.schemata for details.
finalizeATCTSchema(SyncFolderSchema)

class SyncFolder(ATFolder):
    """
    SyncFolder
    """

    # Standard content type setup
    portal_type = meta_type = 'SyncFolder'
    archetype_name = 'SyncFolder'
    #content_icon = 'SyncFolder.gif'
    schema = SyncFolderSchema
    typeDescription= 'SyncFolder'
    typeDescMsgId  = 'SyncFolder'


    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Make sure we get all the interface declarations from ATFolder,
    __implements__ = ATFolder.__implements__ + \
                     (ISyncFolder,)

    # Get the standard actions (tabs)
    actions = ATFolder.actions

    # This method, from ISelectableBrowserDefault, is used to check whether
    # the "Choose content item to use as deafult view" option will be 
    # presented. This makes sense for folders.
    def canSetDefaultPage(self):
        return True

    # enable FTP/WebDAV and friends
    PUT = ATFolder.PUT

    #ファイルを削除するメソッドをオーバライド。
    #その際に、.pscファイルの中に削除されたファイルの
    #ファイル名と削除時間のリストを保存する。
    #
    #ただし、一回削除されたファイルがもう一度追加されて
    #復活した場合、それが起ったことをどうやって検知するか
    #わからなかった。そういう理由で、.pscファイルの中身は
    #厳密には削除されているファイルのリストにはならない。
    #そのへんはPloneSyncツールのほうで対応させることにする。
    def manage_delObjects(self,delFiles):
        ss = self.getRemovedFileList()
        dt = DateTime()
        for d in delFiles:
            ss.append(d + '/' + (str(long(dt)*1000)))
        #実際の削除
        ATFolder.manage_delObjects(self,delFiles)
        #実在するファイルをリストから削除
        sss = []
        ids = self.objectIds()
        for s in ss:
            b = True
            for i in ids:
                if string.find(s,i)==0: # <-- 一致した場合
                    b = False
            if b:
                sss.append(s)
        #重複を削除(古いほう削除)
        ss = []
        for s1 in sss:
            s11 = string.split(s1,'/')
            b = True # s1を残すべき:True、その逆:False
            for s2 in sss:
                s22 = string.split(s2,'/')
                if s11[0] == s22[0]:
                    if s11[1] < s22[1]:
                        b = False
                        break
            if b:
                ss.append(s1)

        self.setRemovedFileList(ss)

#以下内部で使ってるサブメソッド

    # 削除ファイルのリストを.pscファイルからゲット。
    # 返えされる値はファイル名と削除時間(Long)を「/」で
    # 区切ったもののリスト。
    def getRemovedFileList(self):
        try:
            dot_psc = self['.psc']
            if not dot_psc:
                return []
            ssTmp = string.split(dot_psc.data,'\n')
            ss = []
            for s in ssTmp:
                if s != '':
                    ss.append(s)
            return ss
        except:
            return []
        return[]

    # 削除ファイルのリストを.pscファイルにセット。
    # dFilesにはファイル名と削除時間(Long)を「/」で区切ったものの
    # リストをわたすこと。
    def setRemovedFileList(self,dFiles):
        try:
            dot_psc = self['.psc']
            if not dot_psc:
                self.manage_addFile('.psc','','.psc','','plane/text')
                dot_psc = self['.psc']
        except:
            self.manage_addFile('.psc','','.psc','','plane/text')
            dot_psc = self['.psc']
        list = ''
        for f in dFiles:
            list = list + f + '\n'
        dot_psc.data = list

    # idでファイルをゲットするメソッド。
    # 昨日までの俺はバカだったので、こんなメソッドを
    # 作っていた。オブジェクトは辞書として使えるという
    # ことみたいなので、
    # return self[id]
    # でよし。よってこのメソッド意味なし。臥薪嘗胆。
    # しかし、本当の辞書とは違うのか？同じなのか？
    # 確認するべし。
    def getById(self,id):
        o = None
        for c in self.objectValues():
            if c.getId() == id:
                o = c
                break
        return o

registerType(SyncFolder)
